/**
 * @author Kuitos
 * @homepage https://github.com/kuitos/
 * @since 2018-09-03 15:04
 */
var SCRIPT_TAG_REGEX = /<(script)\s+((?!type=('|')text\/ng-template\3).)*?>.*?<\/\1>/gi;
var SCRIPT_SRC_REGEX = /.*\ssrc=('|")(\S+)\1.*/;
var SCRIPT_ENTRY_REGEX = /.*\sentry\s*.*/;
var LINK_TAG_REGEX = /<(link)\s+.*?>/gi;
var STYLE_TYPE_REGEX = /\s+rel=("|')stylesheet\1.*/;
var STYLE_HREF_REGEX = /.*\shref=('|")(\S+)\1.*/;

function hasProtocol(url) {
  return url.startsWith('//') || url.startsWith('http');
}
/**
 * parse the script link from the template
 * @param tpl
 * @param domain
 * @returns {{template: void | string | *, scripts: *[], entry: *}}
 */


export default function processTpl(tpl, domain) {
  var scripts = [];
  var entry = null;
  var template = tpl // change the css link
  .replace(LINK_TAG_REGEX, function (match) {
    var styleType = !!match.match(STYLE_TYPE_REGEX);

    if (styleType) {
      var styleHref = match.match(STYLE_HREF_REGEX);

      if (styleHref) {
        var href = styleHref && styleHref[2];

        if (href && !hasProtocol(href)) {
          var newHref = domain + href;
          return match.replace(href, newHref);
        }
      }
    }

    return match;
  }).replace(SCRIPT_TAG_REGEX, function (match) {
    var matchedScriptEntry = match.match(SCRIPT_ENTRY_REGEX);
    var matchedScriptSrcMatch = match.match(SCRIPT_SRC_REGEX);
    var matchedScriptSrc = matchedScriptSrcMatch && matchedScriptSrcMatch[2];

    if (entry && matchedScriptEntry) {
      throw new SyntaxError('You should not set multiply entry script!');
    } else {
      // append the domain while the script not have an protocol prefix
      if (matchedScriptSrc && !hasProtocol(matchedScriptSrc)) {
        matchedScriptSrc = domain + matchedScriptSrc;
      }

      entry = entry || matchedScriptEntry && matchedScriptSrc;
    }

    if (matchedScriptSrc) {
      scripts.push(matchedScriptSrc);
      return "<!-- script ".concat(matchedScriptSrc, " replaced -->");
    }

    return match;
  });
  scripts = scripts.filter(function (script) {
    // filter empty script
    return !!script;
  });
  return {
    template: template,
    scripts: scripts,
    // set the last script as entry if have not set
    entry: entry || scripts[scripts.length - 1]
  };
}